'****************************************
'* NTSC 640 x 480 x 8bpp-lut            *
'* Original 256x192x8 by Chip Gracey    *
'* Modified by Jim Bagley to be 640x480 *
'****************************************

CON

'---------------------------------------
' I've not got screen res X or Y done yet with variables, but it'll come.

  ResX	= 640
  ResY	= 400

'---------------------------------------

  XSCALER	= (ResX/321)	' pixel scaler 0 for 320 or 256 pixel mode, 1 for 640 or 512 pixel mode
  XSCALER1	= XSCALER+1
  INTERLACED	= (ResY/241)+1	' 1 = non-interlaced ( 192 200 or 240 lines ), 2 = interlaced ( 384,400 or 480 lines )

'---------------------------------------

  s		= 84					'scales DAC output (s = 0..128)
  r		= s * 78 / 128				'adjusts for modulator expansion

  f_color	= 3_579_545.0				'colorburst frequency
  f_scanline	= f_color / 227.5			'scanline frequency
  f_pixel	= f_scanline * float(400<<XSCALER)	'pixel frequency for 400 pixels per scanline

  f_clock	= 50_000_000.0				'clock frequency

  f_xfr		= f_pixel / f_clock * float($7FFF_FFFF)
  f_csc		= f_color / f_clock * float($7FFF_FFFF) * 2.0

DAT		org

'
' Setup
'
		setxfrq ##round(f_xfr)		'set transfer frequency
		setcfrq	##round(f_csc)		'set colorspace converter frequency
		setcy	mody			'set colorspace converter coefficients
		setci	modi
		setcq	modq
		setcmod	#%11_1_0000		'set colorspace converter to YIQ mode (composite)

		rdfast	#0,##$1000-$400		'load .bmp palette into lut
		mov	x,#0
		rep	@.end,#$100
		rflong	y
		shl	y,#8
		wrlut	y,x
		incmod	x,#$100	wc
.end
		rdfast	##640*480/64,##$1000	'set rdfast to wrap on bitmap

'
' Field loop
'
field           mov	x,#34-24+((240-(ResY/INTERLACED))/2)	'top blanks
		call	#blank

		mov	bitmapptr,##$1000

                mov     x,#(ResY/INTERLACED)'set visible lines
line	        call	#hsync			'do horizontal sync
		rdfast	##0,bitmapptr		'set rdfast to wrap on bitmap
		xcont	m_rf,#0			'visible line (rfword rgb16 (5:6:5)
		add	bitmapptr,linelen
		cmp	is_interlaced,#0 wz
	if_nz	add	bitmapptr,linelen
		djnz    x,#line           	'another line?

                mov     x,#28-24+((240-(ResY/INTERLACED))/2)	'bottom blanks
		call	#blank

		tjnz	is_interlaced,#do_interlaced

		mov	x,#6			'high vertical syncs
		call	#vsynch
		mov	x,#6			'low vertical syncs
		call	#vsyncl
		mov	x,#6			'high vertical syncs
		call	#vsynch
		jmp	#field

do_interlaced
		mov	x,#6			'high vertical syncs
		call	#vsynch
		mov	x,#5			'low vertical syncs
		call	#vsyncl
		mov	x,#5			'high vertical syncs
		call	#vsynch

field2          mov	x,#34-24+((240-(ResY/INTERLACED))/2)	'top blanks
		call	#blank

		mov	bitmapptr,##$1000
		cmp	is_interlaced,#0 wz
	if_nz	add	bitmapptr,linelen

                mov     x,#(ResY/INTERLACED)'set visible lines
line2	        call	#hsync			'do horizontal sync
		rdfast	##0,bitmapptr	'set rdfast to wrap on bitmap
		xcont	m_rf,#0			'visible line (rfword rgb16 (5:6:5)
		add	bitmapptr,linelen
		cmp	is_interlaced,#0 wz
	if_nz	add	bitmapptr,linelen
		djnz    x,#line2           	'another line?

                mov     x,#28-24+((240-(ResY/INTERLACED))/2)	'bottom blanks
		call	#blank

		mov	x,#5			'high vertical syncs
		call	#vsynch
		mov	x,#5			'low vertical syncs
		call	#vsyncl
		mov	x,#4			'high vertical syncs
		call	#vsynch
                jmp     #field                  'loop

'
' Subroutines
'
blank		call	#hsync			'blank lines
'		xcont	m_vi,c_vw
		xcont	m_vi,#0
		djnz	x,#blank
		ret

hsync		xcont	m_bs,#1			'horizontal sync
		xcont	m_sn,#2
		xcont	m_bc,#1
		xcont	m_cb,c_cb
		xcont	m_bv,#1
		ret

vsynch		xcont	m_bs,#1			'vertical sync high
		xcont	m_hs,#2
		xcont	m_hl,#1
		djnz	x,#vsynch
		ret

vsyncl		xcont	m_bs,#1			'vertical sync low
		xcont	m_hl,#2
		xcont	m_hs,#1
		djnz	x,#vsyncl
		ret

'
' Initialized data
'
mody		long	((+38*s/128) & $FF) << 24 + ((+75*s/128) & $FF) << 16 + ((+15*s/128) & $FF) << 8 + (110*s/128 & $FF)
modi		long	((+76*r/128) & $FF) << 24 + ((-35*r/128) & $FF) << 16 + ((-41*r/128) & $FF) << 8 + (100*s/128 & $FF)
modq		long	((+27*r/128) & $FF) << 24 + ((-67*r/128) & $FF) << 16 + ((+40*r/128) & $FF) << 8 + 128

m_sn		long	$CF000000+(29<<XSCALER)		'sync
m_bc		long	$CF000000+(7<<XSCALER)		'before colorburst
m_cb		long	$CF000000+(18<<XSCALER)		'colorburst

m_bs		long	$CF000000+(((50-34) + ((320-(ResX/XSCALER1))/2))<<XSCALER)	'before sync
m_bv		long	$CF000000+(((40-30) + ((320-(ResX/XSCALER1))/2))<<XSCALER)	'before visible
m_vi		long	$CF000000+ResX	'visible
m_rf		long	$7F000000+ResX	'visible rflong 8bpp lut

m_hs		long	$CF000000+(20<<XSCALER)		'vertical sync short
m_hl		long	$CF000000+(130<<XSCALER)	'vertical sync long

c_cb		long	$687800_01			'colorburst reference color
c_vw		long	$FFFF00_00			'white
c_vb		long	$000000_00			'black

x		long	0
y		long	0

is_interlaced	long	INTERLACED-1

bitmapptr	long	0
linelen		long	640

'
' Bitmap
'
		orgh	$1000 - $436	'justify pixels at $1000, pallete at $1000-$400

'		file	"bubble.bmp"
'		file	"shells.bmp"
'		file	"testcard2.bmp"
'		file	"640x480.bmp"
'		file	"Batman-Superman1.bmp"
		file	"chip.bmp"
'		file	"win95.bmp"
'		file	"sddefault.bmp"
'		file	"doom.bmp"	'this is originally a 320x240 image scaled to 640x480
